import rovcontrol.drivers.motor.MotorTemp as MotorTemp
import rovcontrol.drivers.abstract.PWMBlock as PWMBlock
import RPi.GPIO as GPIO #TODO: abstract away gpio

class Pololu18v17(MotorTemp.MotorTemp):
    MAX_PWM = 0.9999
    MIN_PWM = 0
    def __init__(self, pwml, dirl, sleepl):
        pwml.set_freq(20000)
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(dirl, GPIO.OUT)
        GPIO.setup(sleepl, GPIO.OUT)
        self.sleepl = sleepl
        self.dirl = dirl
        self.pwml = pwml
        super().__init__(range(0, len(dirl)))

    def update_one_channel(self, channel, data):
        if data['dirty']:
            GPIO.output(self.sleepl[channel], not data['sleep'])
            self.pwml.set_duty(channel, data['speed'] / 100.0 * (self.MAX_PWM - self.MIN_PWM) + self.MIN_PWM)
            GPIO.output(self.dirl[channel], data['dir'])
            if data['brake']:
                self.pwml.set_duty(channel, 0)
        super().update_one_channel(channel, data)



import rovcontrol.drivers.DriverTemp as DriverTemp
import copy

class PWMTemp(DriverTemp.DriverTemp):
    default_bus_settings = {'freq' : 60, 'dirty' : True, 'channels' : {}}
    default_channel_settings = {'polarity' : 0, 'duty' : 0.0, 'dirty' : True}

    def __init__(self):
        self.reglist = [
            {'name' : "freq", 'type' : int, 'depth' : 1,
             'set' : self.set_freq, 'get' : self.get_freq},
            {'name' : "polarity", 'type' : int, 'depth' : 2,
             'set' : self.set_polarity, 'get' : self.get_polarity},
            {'name' : "duty", 'type' : float, 'depth' : 2,
             'set' : self.set_duty, 'get' : self.get_duty}]
        self.settings = {}
    def __str__(self):
        return str(self.settings)

    def enable_bus(self, bus):
        self.settings.update({bus : copy.deepcopy(self.default_bus_settings)})
        self.update()
    def enable_chan(self, bus, channel):
        self.settings[bus]['channels'].update({channel : copy.deepcopy(self.default_channel_settings)})
        self.update()

    def update(self):
        for i in self.settings.keys():
            self.update_one_bus(i, self.settings[i])

    def update_one_bus(self, bus, data):
        data['dirty'] = False
        for i in self.settings[bus]['channels'].keys():
            self.update_one_channel(bus, i, self.settings[bus]['channels'][i])

    def update_one_channel(self, bus, channel, data):
        data['dirty'] = False

    def set_freq(self, bus, freq):
        bus = int(bus)
        self.settings[bus]['freq'] = freq
        self.settings[bus]['dirty'] = True
        self.update()
    def get_freq(self, bus):
        bus = int(bus)
        return self.settings[bus]['freq']
    
    def set_polarity(self, bus, channel, polarity):
        bus = int(bus)
        channel = int(channel)
        self.settings[bus]['channels'][channel]['polarity'] = polarity
        self.settings[bus]['channels'][channel]['dirty'] = True
        self.update()
    def get_polarity(self, bus, channel):
        bus = int(bus)
        channel = int(channel)
        return self.settings[bus]['channels'][channel]['polarity']

    def set_duty(self, bus, channel, duty):
        bus = int(bus)
        channel = int(channel)
        self.settings[bus]['channels'][channel]['duty'] = duty
        self.settings[bus]['channels'][channel]['dirty'] = True
        self.update()
    def get_duty(self, bus, channel):
        bus = int(bus)
        channel = int(channel)
        return self.settings[bus]['channels'][channel]['duty']



import rovcontrol.drivers.DriverTemp as DriverTemp
import copy

class CurrentTemp(DriverTemp.DriverTemp):
    default_channel_settings = {'current' : 0, 'dirty' : True}

    def __init__(self, channels):
        self.settings = {}
        self.reglist = [
            {'name' : "current", type : int, 'depth' : 1,
             'set' : set_current, 'get' : get_current}]
        for i in channels:
            self.settings.update({i : copy.deepcopy(self.default_channel_settings)})
        self.updte()
    def __str__(self):
        return str(self.settings)

    def update(self):
        for i in self.settings.keys():
            self.update_one_channel(i, self.settings[i])

    def update_one_channel(self, channel, data):
        data['dirty'] = False

    def set_current(self, channel, current):
        pass
    def get_current(self, channel):
        channel = int(channel)
        self.settings[channel]['dirty'] = True
        self.update()
        return self.settings[channel]['current']



class DriverTemp:
    def __init__(self):
        self.settings = {}

    def get_reg_list(self):
        return self.reglist

    def get_reg_type(self, regname):
        for i in self.get_reg_list():
            if i['name'] == regname:
                return i['type']

    def get_reg(self, regname, *args):
        for i in self.get_reg_list():
            if i['name'] == regname:
                return i['get'](*args)
    def set_reg(self, regname, value, *args):
        for i in self.get_reg_list():
            if i['name'] == regname:
                return i['set'](*args, self.get_reg_type(regname)(value))



import rovcontrol.drivers.adc.ADCTemp as ADCTemp

class ADCBlock:
    def __init__(self):
        self.settings = {}
    def __str__(self):
        return str(self.settings)

    def map_chan(self, controller, real_bus, real_chan, logical_chan):
        self.settings.update({logical_chan : {'controller' : controller, 'real_bus' : real_bus, 'real_chan' : real_chan}})

    def set_vref(self, vref):
        for i in self.settings.keys():
            self.settings[i]['controller'].set_vref(self.settings[i]['real_bus'], vref)
    def get_vref(self):
        for i in self.settings.keys():
            self.settings[i]['controller'].get_vref(self.settings[i]['real_bus'])
        return 0

    def set_mv(self, logical_chan, mv):
        self.settings[logical_chan]['controller'].set_mv(
            self.settings[logical_chan]['real_bus'], settings[logical_chan]['real_chan'], mv)
    def get_mv(self, logical_chan):
        return self.settings[logical_chan]['controller'].get_mv(
            self.settings[logical_chan]['real_bus'], settings[logical_chan]['real_chan'])



import rovcontrol.drivers.current.CurrentTemp as CurrentTemp
import rovcontrol.drivers.abstract.ADCBlock as ADCBlock

class PololuCurrent(CurrentTemp.CurrentTemp):
    VREF = 5000
    def __init__(self, channels, adcl):
        adcl.set_vref(self.VREF)
        self.adcl = adcl
        super().__init__(channels)

    def scale(self, value):
        return 1000 * (value - 50) / 20.0

    def update_one_channel(self, channel, data):
        if data['dirty']:
            data['current'] = self.scale(adcl.get_mv(channel))
        super().update_one_channel(channel, data)



class ComTemp:
    def __init__(self):
        self.settings = {}
    def __str__(self):
        return str(self.settings)

    def add_driver(self, driver, name):
        self.settings.update({name : driver})

    def getr(self, drivername, regname, *args):
        return self.settings[drivername].get_reg(regname, *args)
    def setr(self, drivername, regname, value, *args):
        return self.settings[drivername].set_reg(regname, value, *args)



import asyncio, json, websockets
import rovcontrol.drivers.com.ComTemp as ComTemp

class WsCom(ComTemp.ComTemp):
    async def handle_con(self, ws, p):
        while True:
            command = await ws.recv()
            command = json.loads(command)
            if command['op'] == "get":
                x = self.getr(command['device'], command['register'], command['channel'])
                command.update({'value' : x})
                await ws.send(json.dumps(command))
            if command['op'] == "set":
                self.setr(command['device'], command['register'], command['value'], command['channel'])

    def listen(self, port = 8000):
        asyncio.get_event_loop().run_until_complete(websockets.serve(self.handle_con, '0.0.0.0', port))
        asyncio.get_event_loop().run_forever()



import rovcontrol.drivers.DriverTemp as DriverTemp
import copy

class ADCTemp(DriverTemp.DriverTemp):
    default_bus_settings = {'vref' : 0, 'dirty' : True, 'channels' : {}}
    default_channel_settings = {'mv' : 0, 'dirty' : True}

    def __init__(self):
        self.reglist = []
        self.settings = {}
    def __str__(self):
        return str(self.settings)

    def enable_bus(self, bus):
        self.settings.update({bus : copy.deepcopy(self.default_bus_settings)})
        self.update()
    def enable_chan(self, bus, channel):
        self.settings[bus]['channels'].update({channel : copy.deepcopy(self.default_channel_settings)})
        self.update()

    def update(self):
        for i in self.settings.keys():
            self.update_one_bus(i, self.settings[i])

    def update_one_bus(self, bus, data):
        data['dirty'] = False
        for i in self.settings[bus]['channels'].keys():
            self.update_one_channel(bus, i, self.settings[bus]['channels'][i])

    def update_one_channel(self, bus, channel, data):
        data['dirty'] = False

    def set_vref(self, bus, vref):
        bus = int(bus)
        self.settings[bus]['vref'] = vref
        self.settings[bus]['dirty'] = True
        self.update()
    def get_vref(self, bus):
        bus = int(bus)
        return self.settings[bus]['vref']

    def set_mv(self, bus, channel, mv):
        pass
    def get_mv(self, bus, channel):
        bus = int(bus)
        channel = int(channel)
        self.settings[bus]['channels'][channel]['dirty'] = True
        self.update()
        return self.settings[bus]['channels'][channel]['mv']



import rovcontrol.drivers.DriverTemp as DriverTemp
import copy

class MotorTemp(DriverTemp.DriverTemp):
    default_channel_settings = {'speed' : 0, 'dir' : 0, 'sleep' : False, 'brake' : False, 'lock' : False, 'dirty' : True}
        
    def __init__(self, channels):
        self.settings = {}
        self.reglist = [
            {'name' : "speed", 'type' : int, 'depth' : 1,
             'set' : self.set_speed, 'get' : self.get_speed},
            {'name' : "speed_fancy", 'type' : int, 'depth' : 1,
             'set' : self.set_speed_fancy, 'get' : self.get_speed_fancy},
            {'name' : "dir", 'type' : int, 'depth' : 1,
             'set' : self.set_dir, 'get' : self.get_dir},
            {'name' : "sleep", 'type' : bool, 'depth' : 1,
             'set' : self.set_sleep, 'get' : self.get_sleep},
            {'name' : "brake", 'type' : bool, 'depth' : 1,
             'set' : self.set_brake, 'get' : self.get_brake},
            {'name' : "lock", 'type' : bool, 'depth' : 1,
             'set' : self.set_lock, 'get' : self.get_lock}]
        for i in channels:
            self.settings.update({i : copy.deepcopy(self.default_channel_settings)})
        self.update()
    def __str__(self):
        return str(self.settings)

    def update(self):
        for i in self.settings.keys():
            self.update_one_channel(i, self.settings[i])

    def update_one_channel(self, channel, data):
        data['dirty'] = False

    def set_speed(self, channel, speed):
        channel = int(channel)
        self.settings[channel]['speed'] = speed
        self.settings[channel]['dirty'] = True
        self.update()
    def get_speed(self, channel):
        channel = int(channel)
        return self.settings[channel]['speed']

    def set_dir(self, channel, direction):
        channel = int(channel)
        self.settings[channel]['dir'] = direction
        self.settings[channel]['dirty'] = True
        self.update()
    def get_dir(self, channel):
        channel = int(channel)
        return self.settings[channel]['dir']

    def set_brake(self, channel, brake):
        channel = int(channel)
        self.settings[channel]['brake'] = brake
        self.settings[channel]['dirty'] = True
        self.update()
    def get_brake(self, channel):
        channel = int(channel)
        return self.settings[channel]['brake']

    def set_lock(self, channel, lock):
        channel = int(channel)
        self.settings[channel]['lock'] = lock
        self.settings[channel]['dirty'] = True
        self.update()
    def get_lock(self, channel):
        channel = int(channel)
        return self.settings[channel]['lock']

    def set_sleep(self, channel, sleep):
        channel = int(channel)
        self.settings[channel]['sleep'] = sleep
        self.settings[channel]['dirty'] = True
        self.update()
    def get_sleep(self, channel):
        channel = int(channel)
        return self.settings[channel]['sleep']

    def set_speed_fancy(self, channel, fancyspeed):
        channel = int(channel)
        self.set_dir(channel, 0 if fancyspeed >= 0 else 1)
        self.set_speed(channel, abs(fancyspeed))
    def get_speed_fancy(self, channel):
        channel = int(channel)
        return self.settings[channel]['speed'] if dir == 0 else self.settings[channel]['speed'] * -1



import rovcontrol.drivers.pwm.PWMTemp as PWMTemp

class PWMBlock:
    def __init__(self):
        self.settings = {}
    def __str__(self):
        return str(self.settings)

    def map_chan(self, controller, real_bus, real_chan, logical_chan):
        self.settings.update({logical_chan : {'controller' : controller, 'real_bus' : real_bus, 'real_chan' : real_chan}})

    def set_freq(self, freq):
        for i in self.settings.keys():
            self.settings[i]['controller'].set_freq(self.settings[i]['real_bus'], freq)
    def set_polarity(self, logical_chan, polarity):
        self.settings[logical_chan]['controller'].set_polarity(
            self.settings[logical_chan]['real_bus'], settings[logical_chan]['real_chan'], polarity)
    def set_duty(self, logical_chan, duty):
        self.settings[logical_chan]['controller'].set_duty(
            self.settings[logical_chan]['real_bus'], self.settings[logical_chan]['real_chan'], duty)



import rovcontrol.drivers.adc.ADCTemp as ADCTemp

import Adafruit_MCP3008
import Adafruit_GPIO.SPI as SPI

class MCP3008(ADCTemp.ADCTemp):
    def __init__(self):
        self.adaadc = Adafruit_MCP3008.MCP3008(spi=SPI.SpiDev(0, 0))
        super().__init__()

    def scale(self, value, bus):
        return (value / 1023) * self.get_vref(bus)

    def update_one_channel(self, bus, channel, data):
        if data['dirty']:
            data['mv'] = self.scale(self.adaadc.read_adc(channel), bus)
        super().update_one_channel(bus, channel, data)



from setuptools import setup, find_packages

setup(
    name='rovcontrol',
    version='0.6.0',
    author='Aled Cuda',
    author_email='aledvirgil@gmail.com',
    url='https://github.com/ld-cd/rovcontrol/',
    packages=find_packages(),
    install_requires=['websockets', 'RPi.GPIO', 'adafruit-pca9685', 'adafruit-mcp3008', 'adafruit-gpio',],
    license='MIT',
    long_description="I'll document it eventually...",
)



import rovcontrol.drivers.com.ComTemp as ComTemp

class DummyCom(ComTemp.ComTemp):
    def listen(self):
        while True:
            op, *arg, x= input("--> ").split(" ")
            if op == "get":
                print(self.getr(arg[0], arg[1], *(arg[2:len(arg)])))
            if op == "set":
                self.setr(arg[0], arg[1], x, *(arg[2:len(arg)]))



import rovcontrol.drivers.pwm.PWMTemp as PWMTemp
import Adafruit_PCA9685

#TODO: implement polarity

class PCA9685(PWMTemp.PWMTemp):
    def __init__(self, i2caddress = 0x40, i2cbus = 1):
        super().__init__()
        self.adapwm = Adafruit_PCA9685.PCA9685(address = i2caddress, busnum = i2cbus)

    def update_one_bus(self, bus, data):
        if data['dirty']:
            self.adapwm.set_pwm_freq(data['freq'])
        super().update_one_bus(bus, data)

    def update_one_channel(self, bus, channel, data):
        if data['dirty']:
            self.adapwm.set_pwm(channel, 0, int(data['duty'] * 4096))
        super().update_one_channel(bus, channel, data)
